;**********************************************************************
;	Serial Communications Interface Subroutines
;	File Name: sercom.asm
;	Date: 09/05/2000
;	Author: John Pavlick, Jr. / I.D.S.
;
;	Processor: Microchip / PIC 14 bit midrange MCU
;	Hardware: DCE or DTE
;	External Defines Required (RAM variables in Bank 0):
;		BAUD_SEL = selects baud rate based on table value 
;		SER_OUT = I/O pin for serial transmit: #define SER_OUT <PORT,BIT> 
;		SER_IN = I/O pin for serial receive: #define SER_IN <PORT,BIT>
;		Txbuff = transmit buffer (1 byte)
;		Rxbuff = receive buffer (1 byte)
;		bit_count = counts # of bits to be transmitted or received
;		wait_cnt = wait loop counter
;
;	Protocol: 8 N 1 NRZ (transmits LSB -> MSB) (receives LSB-> MSB)
;	external constant: BAUD_SEL - selects baud rate constant value
;
;	Notes: 
;	1. [receive:] subroutine waits 1.5 * 1/baudrate after receiving
;	start bit to sample at middle of each data and stop bit period
;	2. For half duplex application only
;
;**********************************************************************

;***** Change BAUD_SEL for baud rate desired:
;***** This can be defined in main module if necessary
;***** BAUD_SEL values for 4 MHz. crystal (1 MHz. internal clock)
;#define BAUD_SEL D'23'				; 9600 bps
;#define BAUD_SEL D'49'				; 4800 bps
;#define BAUD_SEL D'101'			; 2400 bps
;#define BAUD_SEL D'205'			; 1200 bps

;***** SUBROUTINES

;**********************************************************************
;***** SERIAL TRANSMIT ROUTINE [transmit:]
;***** Call with data to transmit in W-reg. / sends LSB first on SER_OUT (I/O pin)
;***** Sets 'C' bit after completion if no errors

transmit:	bcf	STATUS,RP0		; select Bank 0
		movwf	Txbuff	  		; put data byte into temporary buffer (to preserve original data)
		movlw	D'9'			; transmit 9 bits (+ stop bit)
		movwf	bit_count		; initialize bit counter
start_bit	bcf	SER_OUT			; send start bit '0'
		nop				; pad for timing
		bsf	STATUS,C		; set 'C' for stop bit (& frame error check)
bit_delay	movlw	BAUD_SEL		; get bit time constant
		movwf	wait_cnt		; initialize counter 
bit_delay1	nop				; makes a 4 cycle loop
		decfsz	wait_cnt,F		; countdown
		goto	$-2	  		; loop until time up
		nop				; pad for timing
next_bit	decfsz	bit_count,F		; all 9 bits transmitted ?
		goto	send_data		; no, go and transmit next bit
		goto	$+1			; pad for timing (2 cycles)
stop_bit	rrf	Txbuff,F		; rotate to get the stop bit
		btfss	STATUS,C		; see if 'C' is set
		goto	transmit_err		; will exit with an error flag (C=0)
		bsf	SER_OUT			; send stop '1'
stop_bit1	movlw	BAUD_SEL+2		; get bit time constant (+ 8 cycles)
		movwf	wait_cnt		; initialize counter
		nop				; makes a 4 cycle loop
		decfsz	wait_cnt,F
		goto	$-2			; loop until time over
		goto	transmitx		; go to exit (with 'C' set)
send_data	rrf	Txbuff,F		; rotate to put next bit in 'C'
		btfsc	STATUS,C		; see if 'C' is clear
		goto	transmit_hi		; go send a '1'
		nop				; pad for timing
transmit_lo	bcf	SER_OUT			; send a '0'
		goto	bit_delay		; wait for bit time
transmit_hi	bsf	SER_OUT			; send a '1'
		goto	bit_delay		; wait for bit time
transmit_err	bsf	SER_OUT			; make sure send line is in idle state
transmitx	return				; exit

;**********************************************************************
;***** SERIAL RECEIVE ROUTINE [receive:]
;***** Receives data from SER_IN (I/O pin) LSB first returns data in 'W' reg.
;***** Frame error: 'C'=0 , no errors: 'C'=1

receive:	bcf	STATUS,RP0		; bank 0
		btfsc	SER_IN			; see if start bit = 0
		goto	Rxerror			; Rx line idle
receive1	movlw	D'9'			; get 9 bits...
		movwf	bit_count		; initialize counter
		movlw	BAUD_SEL/2		; get time offset for start bit
		movwf 	wait_cnt		; initialize counter
start_wait	nop				; makes a 4 cycle loop
		decfsz	wait_cnt,F
		goto	start_wait		; loop until start bit offset time is over
		nop				; makes last iteration 4 cycles
		clrf	Rxbuff			; clear the input buffer
		bcf	STATUS,C		; make sure 'C' is clear or will write into buffer
get_bit		goto	$+1			; pad for timing (2 cycles)
		movlw	BAUD_SEL		; get time constant
		movwf	wait_cnt		; initialize counter
data_wait	nop				; makes a 4 cycle loop
		decfsz	wait_cnt,F
		goto	data_wait		; loop until time is over
		nop				; makes last iteration 4 cycles
		nop				; pad for timing
read_bit	bsf	Rxbuff,0		; assume a 1
		btfss	SER_IN			; read the actual state
		bcf	Rxbuff,0		; make a 0
		rrf	Rxbuff,F		; NEW_BIT -> 'C'
		decfsz	bit_count,F		; on last bit ?
		goto 	get_bit	  		; no, get next bit
got_stop	btfss	STATUS,C		; should be a '1'
		goto	Rxerror			; return an error flag
		movf	Rxbuff,W		; put return value in W-reg.
		goto	receivex		; go to exit
Rxerror		bcf	STATUS,C		; return an error flag (C=0)
receivex	return				; exit

;**********************************************************************

